@@ -68,7 +68,7 @@ long long atoll_b (const char *nptr);
 /* Prototypes for worker functions.  */
 int hogcpu (void);
 int hogio (void);
-int hogvm (long long bytes, int hang);
+int hogvm (long long bytes, long long stride, long long hang);
 int hoghdd (long long bytes, int clean);
 
 int
@@ -85,7 +85,8 @@ main (int argc, char **argv)
   long long do_io = 0;
   long long do_vm = 0;
   long long do_vm_bytes = 256 * 1024 * 1024;
-  int do_vm_hang = 0;
+  long long do_vm_stride = 4096;
+  long long do_vm_hang = -1;
   long long do_hdd = 0;
   int do_hdd_clean = 2;
   long long do_hdd_bytes = 1024 * 1024 * 1024;
@@ -190,9 +191,25 @@ main (int argc, char **argv)
               exit (1);
             }
         }
+      else if (strcmp (arg, "--vm-stride") == 0)
+        {
+          assert_arg ("--vm-stride");
+          do_vm_stride = atoll_b (arg);
+          if (do_vm_stride <= 0)
+            {
+              err (stderr, "invalid stride value: %lli\n", do_vm_stride);
+              exit (1);
+            }
+        }
       else if (strcmp (arg, "--vm-hang") == 0)
         {
-          do_vm_hang = 1;
+          assert_arg ("--vm-hang");
+          do_vm_hang = atoll_b (arg);
+          if (do_vm_hang < 0)
+            {
+              err (stderr, "invalid value: %lli\n", do_vm_hang);
+              exit (1);
+            }
         }
       else if (strcmp (arg, "--hdd") == 0 || strcmp (arg, "-d") == 0)
         {
@@ -264,7 +281,7 @@ main (int argc, char **argv)
             }
           else
             {
-              dbg (stdout, "used up time before all workers dispatched\n");
+              wrn (stderr, "used up time before all workers dispatched\n");
               break;
             }
         }
@@ -319,7 +336,7 @@ main (int argc, char **argv)
               usleep (backoff);
               if (do_dryrun)
                 exit (0);
-              exit (hogvm (do_vm_bytes, do_vm_hang));
+              exit (hogvm (do_vm_bytes, do_vm_stride, do_vm_hang));
             case -1:           /* error */
               err (stderr, "fork failed: %s\n", strerror (errno));
               break;
@@ -455,7 +472,7 @@ hogio ()
 }
 
 int
-hogvm (long long bytes, int hang)
+hogvm (long long bytes, long long stride, long long hang)
 {
   long long i;
   char *ptr;
@@ -463,9 +480,10 @@ hogvm (long long bytes, int hang)
   while (1)
     {
       dbg (stdout, "allocating %lli bytes ...\n", bytes);
+      dbg (stdout, "striding by %lli bytes ...\n", stride);
       if ((ptr = (char *) malloc (bytes * sizeof (char))))
         {
-          for (i = 0; i < bytes; i++)
+          for (i = 0; i < bytes; i += stride)
             ptr[i] = 'Z';       /* Ensure that COW happens.  */
         }
       else
@@ -474,12 +492,17 @@ hogvm (long long bytes, int hang)
           return 1;
         }
 
-      if (hang)
+      if (hang == 0)
         {
           dbg (stdout, "sleeping forever with allocated memory\n");
           while (1)
             sleep (1024);
         }
+      else if (hang > 0)
+        {
+          dbg (stdout, "sleeping for %llis with allocated memory\n", hang);
+          sleep (hang);
+        }
 
       free (ptr);
       dbg (stdout, "freed %lli bytes\n", bytes);
@@ -702,21 +725,22 @@ usage (int status)
   char *mesg =
     "`%s' imposes certain types of compute stress on your system\n\n"
     "Usage: %s [OPTION [ARG]] ...\n"
-    " -?, --help            show this help statement\n"
-    "     --version         show version statement\n"
-    " -v, --verbose         be verbose\n"
-    " -q, --quiet           be quiet\n"
-    " -n, --dry-run         show what would have been done\n"
-    " -t, --timeout N       timeout after N seconds\n"
-    "     --backoff N       wait factor of N microseconds before work starts\n"
-    " -c, --cpu N           spawn N workers spinning on sqrt()\n"
-    " -i, --io N            spawn N workers spinning on sync()\n"
-    " -m, --vm N            spawn N workers spinning on malloc()/free()\n"
-    "     --vm-bytes B      malloc B bytes per vm worker (default is 256MB)\n"
-    "     --vm-hang         do not free memory allocated by vm workers\n"
-    " -d, --hdd N           spawn N workers spinning on write()/unlink()\n"
-    "     --hdd-bytes B     write B bytes per hdd worker (default is 1GB)\n"
-    "     --hdd-noclean     do not unlink files created by hdd workers\n\n"
+    " -?, --help         show this help statement\n"
+    "     --version      show version statement\n"
+    " -v, --verbose      be verbose\n"
+    " -q, --quiet        be quiet\n"
+    " -n, --dry-run      show what would have been done\n"
+    " -t, --timeout N    timeout after N seconds\n"
+    "     --backoff N    wait factor of N microseconds before work starts\n"
+    " -c, --cpu N        spawn N workers spinning on sqrt()\n"
+    " -i, --io N         spawn N workers spinning on sync()\n"
+    " -m, --vm N         spawn N workers spinning on malloc()/free()\n"
+    "     --vm-bytes B   malloc B bytes per vm worker (default is 256MB)\n"
+    "     --vm-stride B  touch a byte every B bytes (default is 4096)\n"
+    "     --vm-hang N    sleep N secs before free (default is none, 0 is inf)\n"
+    " -d, --hdd N        spawn N workers spinning on write()/unlink()\n"
+    "     --hdd-bytes B  write B bytes per hdd worker (default is 1GB)\n"
+    "     --hdd-noclean  do not unlink files created by hdd workers\n\n"
     "Example: %s --cpu 8 --io 4 --vm 2 --vm-bytes 128M --timeout 10s\n\n"
     "Note: Numbers may be suffixed with s,m,h,d,y (time) or B,K,M,G (size).\n";
 
